# -*- coding: utf-8 -*-

from __future__ import unicode_literals

__version__ = '0.10.0'
request_faq_category_identifier = 'aldryn_faq_current_category'
request_faq_question_identifier = 'aldryn_faq_current_question'



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf import settings
from django.template import RequestContext

from aldryn_search.utils import get_index_base, strip_tags

from .models import Question, Category


class QuestionIndex(get_index_base()):

    haystack_use_for_indexing = getattr(
        settings, "ALDRYN_FAQ_QUESTION_SEARCH", True)

    INDEX_TITLE = True

    def get_title(self, obj):
        return obj.title

    def get_index_kwargs(self, language):
        return {'translations__language_code': language}

    def get_index_queryset(self, language):
        return self.get_model().objects.all()

    def get_model(self):
        return Question

    def get_search_data(self, obj, language, request):
        text_bits = [strip_tags(obj.title), strip_tags(obj.answer_text)]
        plugins = obj.answer.cmsplugin_set.filter(language=language)
        for base_plugin in plugins:
            instance, plugin_type = base_plugin.get_plugin_instance()
            if instance is not None:
                plugin_content = strip_tags(
                    instance.render_plugin(context=RequestContext(request)))
                text_bits.append(plugin_content)

        return ' '.join(text_bits)


class CategoryIndex(get_index_base()):

    haystack_use_for_indexing = getattr(
        settings, "ALDRYN_FAQ_CATEGORY_SEARCH", True)

    INDEX_TITLE = True

    def get_title(self, obj):
        return ''

    def get_index_kwargs(self, language):
        return {'translations__language_code': language}

    def get_index_queryset(self, language):
        return self.get_model().objects.all()

    def get_model(self):
        return Category

    def get_search_data(self, obj, language, request):
        return strip_tags(obj.name)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_faq.sitemaps import FAQCategoriesSitemap, FAQQuestionsSitemap

from . import AldrynFaqTest


class TestSitemap(AldrynFaqTest):

    def test_categories_sitemap_items(self):
        categories = [cat.id for cat in FAQCategoriesSitemap().items()]
        self.assertEqualItems(categories,
            [self.category1.id, self.category2.id])

    def test_questions_sitemap_items(self):
        questions = [que.id for que in FAQQuestionsSitemap().items()]
        self.assertEqualItems(questions,
            [self.question1.id, self.question2.id])



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.test.client import RequestFactory

from aldryn_faq.views import FaqByCategoryView, FaqAnswerView

from . import AldrynFaqTest, CMSRequestBasedTest


class TestFaqByCategoryView(AldrynFaqTest, CMSRequestBasedTest):
    def test_as_view(self):
        """Tests that the FaqByCategoryView produces the correct context."""
        category1 = self.reload(self.category1, "en")
        question1 = self.reload(self.question1, "en")
        kwargs = {"category_slug": category1.slug}
        category1_url = reverse('aldryn_faq:faq-category', kwargs=kwargs)
        factory = RequestFactory()
        request = factory.get(category1_url)
        response = FaqByCategoryView.as_view()(request, **kwargs)
        self.assertEqualItems(
            response.context_data['object_list'],
            [question1, ],
        )


class TestFaqAnswerView(AldrynFaqTest, CMSRequestBasedTest):
    def test_as_view(self):
        """Tests that the FaqAnswerView produces the correct context."""
        category1 = self.reload(self.category1, "en")
        question1 = self.reload(self.question1, "en")
        kwargs = {"category_slug": category1.slug, "pk": question1.id}
        url = reverse('aldryn_faq:faq-answer', kwargs=kwargs)
        factory = RequestFactory()
        request = factory.get(url)
        response = FaqAnswerView.as_view()(request, **kwargs)
        self.assertEqual(
            response.context_data['object'],
            question1,
        )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django import forms

from parler.forms import TranslatableModelForm

from sortedm2m.forms import SortedMultipleChoiceField

from .models import Category, QuestionListPlugin, Question


class CategoryAdminForm(TranslatableModelForm):

    class Meta:
        model = Category

    # def clean_slug(self):
    #     slug = self.cleaned_data['slug']
    #     translations_model = Category._meta.translations_model
    #     categories_with_slug = translations_model.objects.filter(slug=slug)

    #     if self.instance.pk:
    #         # Make sure to exclude references from this master :)
    #         categories_with_slug = categories_with_slug.exclude(
    #             master_id=self.instance.pk)

    #     if categories_with_slug.exists():
    #         raise forms.ValidationError(
    #             'A category with this slug already exists.')
    #     return slug


class QuestionListPluginForm(forms.ModelForm):

    questions = SortedMultipleChoiceField(queryset=Question.objects.none())

    class Meta:
        model = QuestionListPlugin

    def __init__(self, *args, **kwargs):
        super(QuestionListPluginForm, self).__init__(*args, **kwargs)
        questions_field = self.fields['questions']
        questions_field.queryset = Question.objects.language()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.contenttypes.models import ContentType
from django.utils.encoding import force_text
from django.utils.translation import override

from aldryn_faq.models import Category, Question, get_slug_in_language

from . import AldrynFaqTest


class TestCategory(AldrynFaqTest):

    def test_unicode(self):
        with override('en'):
            category1 = self.reload(self.category1)
            self.assertEqual(
                force_text(category1), self.data["category1"]["en"]["name"])
        with override('de'):
            category1 = self.reload(self.category1)
            self.assertEqual(
                force_text(category1), self.data["category1"]["de"]["name"])

    def test_get_slug_in_language(self):
        self.assertIsNone(get_slug_in_language(None, 'en'), None)
        self.assertIsNone(get_slug_in_language(object, 'en'), None)
        self.assertEqual(
            get_slug_in_language(self.category1, 'en'),
            self.data["category1"]["en"]["slug"]
        )
        self.assertEqual(
            get_slug_in_language(self.category1, 'de'),
            self.data["category1"]["de"]["slug"]
        )
        # Test non-existent translation
        self.assertEqual(
            get_slug_in_language(self.category1, 'qq'),
            None
        )

    def test_model_type_id(self):
        ct = ContentType.objects.get(app_label='aldryn_faq', model='category')
        self.assertEqual(
            self.category1.model_type_id(),
            ct.id
        )

    def test_get_absolue_url(self):
        self.assertEqual(
            self.category1.get_absolute_url("en"),
            "/en/faq/example/"
        )
        self.assertEqual(
            self.category1.get_absolute_url("de"),
            "/de/faq/beispiel/"
        )

    def test_manager_get_categories(self):
        # Test case when no language is passed. Apparently, this returns only
        # those objects that have the default language translations.
        # TODO: Verify that this is the intended behavior.
        # self.assertEqualItems(
        #     [c.id for c in Category.objects.get_categories()],
        #     [self.category1.id, ]
        # )

        # Test case of requesting objects of only a single language. In our
        # setup, we have 1 category in EN and 2 in DE.
        categories = Category.objects.get_categories('en')
        self.assertEqualItems(
            [category.id for category in categories],
            [self.category1.id, ]
        )

        # There's actually two categories in DE
        categories = Category.objects.get_categories('de')
        cids = set([category.id for category in categories])
        self.assertEqual(
            cids,
            set([self.category1.id, self.category2.id])
        )


class TestQuestion(AldrynFaqTest):

    def test_unicode(self):
        with override('en'):
            question1 = self.reload(self.question1)
            self.assertEqual(
                force_text(question1),
                self.data["question1"]["en"]["title"]
            )
        with override('de'):
            question1 = self.reload(self.question1)
            self.assertEqual(
                force_text(question1),
                self.data["question1"]["de"]["title"]
            )

    def test_model_type_id(self):
        ct = ContentType.objects.get(app_label='aldryn_faq', model='question')
        self.assertEqual(
            self.question1.model_type_id(),
            ct.id
        )

    def test_get_absolue_url(self):
        self.assertEqual(
            self.question1.get_absolute_url("en"),
            "/en/faq/example/1/"
        )
        self.assertEqual(
            self.question1.get_absolute_url("de"),
            "/de/faq/beispiel/1/"
        )

    def test_manager_filter_by_language(self):
        questions = Question.objects.filter_by_language('en')
        self.assertEqualItems(
            [q.id for q in questions],
            [self.question1.id]
        )

        questions = Question.objects.filter_by_language('de')
        self.assertEqualItems(
            [q.id for q in questions],
            [self.question1.id, self.question2.id]
        )

    def test_manager_filter_by_current_language(self):
        with override("en"):
            questions = Question.objects.filter_by_current_language()
            self.assertEqualItems(
                [q.id for q in questions],
                [self.question1.id]
            )

        with override("de"):
            questions = Question.objects.filter_by_current_language()
            self.assertEqualItems(
                [q.id for q in questions],
                [self.question1.id, self.question2.id]
            )


class TestFAQTranslations(AldrynFaqTest):

    def test_fetch_faq_translations(self):
        """Test we can fetch arbitrary translations of the question and
        its category."""
        # Can we target the EN values?
        with override("en"):
            question1 = self.reload(self.question1)
            category1 = self.reload(self.question1.category)
            self.assertEqual(
                question1.title,
                self.data["question1"]["en"]["title"]
            )
            self.assertEqual(
                question1.answer_text,
                self.data["question1"]["en"]["answer_text"]
            )
            self.assertEqual(
                category1.name,
                self.data["category1"]["en"]["name"]
            )
            self.assertEqual(
                category1.slug,
                self.data["category1"]["en"]["slug"]
            )

        # And the DE values?
        with override("de"):
            question1 = self.reload(self.question1)
            category1 = self.reload(self.question1.category)
            self.assertEqual(
                question1.title,
                self.data["question1"]["de"]["title"]
            )
            self.assertEqual(
                question1.answer_text,
                self.data["question1"]["de"]["answer_text"]
            )
            self.assertEqual(
                category1.name,
                self.data["category1"]["de"]["name"]
            )
            self.assertEqual(
                category1.slug,
                self.data["category1"]["de"]["slug"]
            )



# -*- coding: utf-8 -*-
import datetime
from south.db import db
from south.v2 import SchemaMigration
from django.db import models


class Migration(SchemaMigration):

    def forwards(self, orm):
        # Adding model 'CategoryTranslation'
        db.create_table(u'aldryn_faq_category_translation', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('name', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('language_code', self.gf('django.db.models.fields.CharField')(max_length=15, db_index=True)),
            ('master', self.gf('django.db.models.fields.related.ForeignKey')(related_name='translations', null=True, to=orm['aldryn_faq.Category'])),
        ))
        db.send_create_signal(u'aldryn_faq', ['CategoryTranslation'])

        # Adding unique constraint on 'CategoryTranslation', fields ['language_code', 'master']
        db.create_unique(u'aldryn_faq_category_translation', ['language_code', 'master_id'])

        # Adding model 'Category'
        db.create_table(u'aldryn_faq_category', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
        ))
        db.send_create_signal(u'aldryn_faq', ['Category'])

        # Adding model 'Question'
        db.create_table(u'aldryn_faq_question', (
            (u'id', self.gf('django.db.models.fields.AutoField')(primary_key=True)),
            ('order', self.gf('django.db.models.fields.PositiveIntegerField')(default=1, db_index=True)),
            ('title', self.gf('django.db.models.fields.CharField')(max_length=255)),
            ('language', self.gf('django.db.models.fields.CharField')(max_length=5)),
            ('category', self.gf('adminsortable.fields.SortableForeignKey')(to=orm['aldryn_faq.Category'])),
            ('answer', self.gf('django.db.models.fields.related.ForeignKey')(related_name='faq_questions', null=True, to=orm['cms.Placeholder'])),
        ))
        db.send_create_signal(u'aldryn_faq', ['Question'])


    def backwards(self, orm):
        # Removing unique constraint on 'CategoryTranslation', fields ['language_code', 'master']
        db.delete_unique(u'aldryn_faq_category_translation', ['language_code', 'master_id'])

        # Deleting model 'CategoryTranslation'
        db.delete_table(u'aldryn_faq_category_translation')

        # Deleting model 'Category'
        db.delete_table(u'aldryn_faq_category')

        # Deleting model 'Question'
        db.delete_table(u'aldryn_faq_question')


    models = {
        u'aldryn_faq.category': {
            'Meta': {'object_name': 'Category'},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'})
        },
        u'aldryn_faq.categorytranslation': {
            'Meta': {'unique_together': "[('language_code', 'master')]", 'object_name': 'CategoryTranslation', 'db_table': "u'aldryn_faq_category_translation'"},
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language_code': ('django.db.models.fields.CharField', [], {'max_length': '15', 'db_index': 'True'}),
            'master': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'translations'", 'null': 'True', 'to': u"orm['aldryn_faq.Category']"}),
            'name': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        u'aldryn_faq.question': {
            'Meta': {'ordering': "['order']", 'object_name': 'Question'},
            'answer': ('django.db.models.fields.related.ForeignKey', [], {'related_name': "'faq_questions'", 'null': 'True', 'to': "orm['cms.Placeholder']"}),
            'category': ('adminsortable.fields.SortableForeignKey', [], {'to': u"orm['aldryn_faq.Category']"}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'language': ('django.db.models.fields.CharField', [], {'max_length': '5'}),
            'order': ('django.db.models.fields.PositiveIntegerField', [], {'default': '1', 'db_index': 'True'}),
            'title': ('django.db.models.fields.CharField', [], {'max_length': '255'})
        },
        'cms.placeholder': {
            'Meta': {'object_name': 'Placeholder'},
            'default_width': ('django.db.models.fields.PositiveSmallIntegerField', [], {'null': 'True'}),
            u'id': ('django.db.models.fields.AutoField', [], {'primary_key': 'True'}),
            'slot': ('django.db.models.fields.CharField', [], {'max_length': '50', 'db_index': 'True'})
        }
    }

    complete_apps = ['aldryn_faq']


# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import reverse
from django.utils.translation import ugettext_lazy as _

from cms.toolbar_pool import toolbar_pool
from cms.toolbar_base import CMSToolbar

from aldryn_faq import (
    request_faq_category_identifier,
    request_faq_question_identifier,
)


@toolbar_pool.register
class FaqToolbar(CMSToolbar):

    def populate(self):

        def can(action, model):
            perm = 'aldryn_faq.%(action)s_%(model)s' % {
                'action': action, 'model': model}
            return self.request.user.has_perm(perm)

        if self.is_current_app and (
                can('add', 'category') or can('change', 'category')):
            menu = self.toolbar.get_or_create_menu('faq-app', _('FAQ'))

            if can('add', 'category'):
                menu.add_modal_item(
                    _('Add category'),
                    reverse('admin:aldryn_faq_category_add')
                )

            category = getattr(self.request,
                request_faq_category_identifier, None)

            if category and can('add', 'question'):
                params = ('?category=%s&language=%s' %
                          (category.pk, self.request.LANGUAGE_CODE))
                menu.add_modal_item(
                    _('Add question'),
                    reverse('admin:aldryn_faq_question_add') + params
                )
            if category and can('change', 'category'):
                url = reverse(
                    'admin:aldryn_faq_category_change',
                    args=(category.pk,),
                )
                menu.add_modal_item(_('Edit category'), url, active=True)

            question = getattr(
                self.request, request_faq_question_identifier, None)

            if question and can('change', 'question'):
                url = reverse(
                    'admin:aldryn_faq_question_change', args=(question.pk,))
                menu.add_modal_item(_('Edit question'), url, active=True)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import (
    get_language_from_request,
    ugettext_lazy as _,
)

from cms.menu_bases import CMSAttachMenu

from menus.base import NavigationNode
from menus.menu_pool import menu_pool

from .models import Category


class FaqCategoryMenu(CMSAttachMenu):

    name = _('FAQ')

    def get_nodes(self, request):
        nodes = []
        lang = get_language_from_request(request, check_path=True)
        categories = Category.objects.translated(lang)

        for category in categories:
            node = NavigationNode(
                category.safe_translation_getter('name', language_code=lang),
                category.get_absolute_url(language=lang),
                category.safe_translation_getter('slug', language_code=lang),
            )
            nodes.append(node)
        return nodes

menu_pool.register_menu(FaqCategoryMenu)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.core.urlresolvers import resolve
from django.db import models
from django.shortcuts import get_list_or_404
from django.utils.translation import get_language_from_request
from django.views.generic import DetailView
from django.views.generic.list import ListView

from menus.utils import set_language_changer

from . import request_faq_category_identifier, request_faq_question_identifier
from .models import Category, Question


class FaqMixin(object):

    model = Question

    def dispatch(self, request, *args, **kwargs):
        self.current_language = get_language_from_request(
            self.request, check_path=True)
        return super(FaqMixin, self).dispatch(request, *args, **kwargs)

    def get_context_data(self, **kwargs):
        context = super(FaqMixin, self).get_context_data(**kwargs)
        context['current_app'] = resolve(self.request.path).namespace
        return context

    def get_queryset(self):
        return self.model.objects.language(self.current_language)


class FaqByCategoryView(FaqMixin, ListView):

    template_name = 'aldryn_faq/questiontranslation_list.html'

    def get(self, *args, **kwargs):
        self.category = self.get_category()
        setattr(self.request, request_faq_category_identifier, self.category)
        response = super(FaqByCategoryView, self).get(*args, **kwargs)
        set_language_changer(self.request, self.category.get_absolute_url)
        return response

    def get_category(self):
        return Category.objects.translated(
            slug=self.kwargs['category_slug']
        )[0]

    def get_queryset(self):
        queryset = super(FaqByCategoryView, self).get_queryset()
        return queryset.filter(category=self.category).order_by('order')


class FaqAnswerView(FaqMixin, DetailView):

    template_name = 'aldryn_faq/question_detail.html'

    def get(self, *args, **kwargs):
        question = self.get_object()
        if hasattr(self.request, 'toolbar'):
            self.request.toolbar.set_object(question)
        setattr(
            self.request, request_faq_category_identifier, question.category)
        setattr(self.request, request_faq_question_identifier, question)
        response = super(FaqAnswerView, self).get(*args, **kwargs)

        # FIXME: We should check for unique visitors using sessions.
        # update number of visits
        question_only_queryset = self.get_queryset().filter(pk=question.pk)
        question_only_queryset.update(
            number_of_visits=models.F('number_of_visits') + 1)

        return response



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import ugettext_lazy as _

from cms.app_base import CMSApp
from cms.apphook_pool import apphook_pool

from .menu import FaqCategoryMenu


class FaqApp(CMSApp):
    name = _('FAQ')
    urls = ['aldryn_faq.urls']
    menus = [FaqCategoryMenu]
    app_name = 'aldryn_faq'

apphook_pool.register(FaqApp)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.utils.translation import get_language

from parler.managers import TranslatableManager


class RelatedManager(TranslatableManager):

    def filter_by_language(self, language):
        return self.active_translations(language)

    def filter_by_current_language(self):
        return self.filter_by_language(get_language())


class CategoryManager(TranslatableManager):

    def get_categories(self, language=None):
        categories = self.active_translations(language).prefetch_related('questions')

        for category in categories:
            category.count = (category.questions
            .filter_by_language(language).count())
        return sorted(categories, key=lambda x: -x.count)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.conf.urls import patterns, url

from .views import FaqByCategoryView, FaqAnswerView


urlpatterns = patterns('',
    url(
        r'^(?P<category_slug>[-\w]+)/$',
        FaqByCategoryView.as_view(),
        name='faq-category'
    ),
    url(
        r'^(?P<category_slug>[-\w]+)/(?P<pk>[0-9]+)/$',
        FaqAnswerView.as_view(),
        name='faq-answer'
    ),
)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.sitemaps import Sitemap

from ..models import Category, Question


class FAQCategoriesSitemap(Sitemap):
    changefreq = "never"
    priority = 0.5

    def items(self):
        return Category.objects.all()


class FAQQuestionsSitemap(Sitemap):
    changefreq = "yearly"
    priority = 0.5

    def items(self):
        return Question.objects.all()



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from distutils.version import LooseVersion

from django.conf import settings
from django.core.urlresolvers import reverse
from django.contrib import admin
from django.templatetags.static import static
from django.utils.translation import get_language

import cms
from cms.admin.placeholderadmin import PlaceholderAdmin
from cms.admin.placeholderadmin import FrontendEditableAdminMixin

from adminsortable.admin import SortableAdmin

from parler.admin import TranslatableAdmin

from .models import Category, Question
from .forms import CategoryAdminForm


class AllTranslationsAdminMixin(object):
    """To use this, apply this mixin to your Admin class, then add
    'all_translations' to your list_display list."""

    def all_translations(self, obj):
        """This is an adapter for the functionality that was in HVAD."""
        available = list(obj.get_available_languages())
        langs = []
        for lang, _ in settings.LANGUAGES:
            if lang in available:
                langs.append(lang)
                available.remove(lang)
        langs += available
        for idx, lang in enumerate(langs):
            change_form_url = reverse('admin:{app_label}_{model_name}_change'.format(
                app_label=obj._meta.app_label.lower(),
                model_name=obj.__class__.__name__.lower(),
            ), args=(obj.id, ))
            link = '<a href="{url}?language={lang}">{lang}</a>'.format(
                url=change_form_url,
                lang=lang,
            )
            if lang == get_language():
                link = "<strong>{0}</strong>".format(link)
            langs[idx] = link
        return ', '.join(langs)
    all_translations.short_description = 'available translations'
    all_translations.allow_tags = True


class CategoryAdmin(AllTranslationsAdminMixin, TranslatableAdmin):

    list_display = ['__str__', 'all_translations']

    form = CategoryAdminForm

    _fieldsets = [(None, {'fields': ['name', 'slug']})]

    class Media:
        # Django BUG - Django only checks for self.prepopulated_fields to
        # determine if it should include these files. But it never checks
        # get_prepopulated_fields()
        js = [static('admin/js/%s' % url) for url in (
            'urlify.js', 'prepopulate.min.js')]

    def get_prepopulated_fields(self, request, obj=None):
        return {'slug': ['name']}

    def get_fieldsets(self, request, obj=None):
        return self._fieldsets


class QuestionAdmin(FrontendEditableAdminMixin, SortableAdmin,
                    PlaceholderAdmin, TranslatableAdmin):

    render_placeholder_language_tabs = False
    list_display = ['__str__', 'category', 'is_top', 'number_of_visits']
    list_filter = ['category', 'translations__language_code']
    frontend_editable_fields = ('title', 'category', 'answer_text')
    readonly_fields = ['number_of_visits']

    def get_fieldsets(self, request, obj=None):
        fieldsets = [
            (None, {
                'fields': [
                    'title', 'category', 'answer_text', 'is_top',
                    'number_of_visits']
            })
        ]
        cms_compat_fieldset = {
            'classes': ['plugin-holder', 'plugin-holder-nopage'],
            'fields': ['answer']
        }

        # show placeholder field if not CMS 3.0
        if LooseVersion(cms.__version__) < LooseVersion('3.0'):
            fieldsets.append(('Answer', cms_compat_fieldset))
        return fieldsets


admin.site.register(Category, CategoryAdmin)
admin.site.register(Question, QuestionAdmin)



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_faq.menu import FaqCategoryMenu
from django.utils.translation import (
    get_language_from_request,
)
from . import AldrynFaqTest, CMSRequestBasedTest


class TestMenu(AldrynFaqTest, CMSRequestBasedTest):

    def test_get_nodes(self):
        # Test that the EN version of the menu has only category1 and is shown
        # in English.
        request = self.get_page_request(None, self.user, '/en/')
        menu = FaqCategoryMenu()
        category1 = self.reload(self.category1, 'en')
        self.assertEqualItems(
            [menuitem.title for menuitem in menu.get_nodes(request)],
            [category1.name]
        )

        # Test that the DE version has 2 categories and that they are shown in
        # German.
        request = self.get_page_request(None, self.user, '/de/')
        menu = FaqCategoryMenu()
        category1 = self.reload(self.category1, 'de')
        category2 = self.reload(self.category2, 'de')
        nodes = menu.get_nodes(request)
        self.assertEqualItems(
            [menuitem.title for menuitem in nodes],
            [category1.name, category2.name]
        )



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from aldryn_faq.search_indexes import QuestionIndex, CategoryIndex

from . import AldrynFaqTest


class TestQuestionIndex(AldrynFaqTest):
    def test_get_title(self):
        idx_obj = QuestionIndex()
        question1 = self.reload(self.question1, "en")
        self.assertEqual(idx_obj.get_title(question1), question1.title)

    def test_get_index_kwargs(self):
        # This is a silly test, but is here for completeness.
        idx_obj = QuestionIndex()
        self.assertEqual(idx_obj.get_index_kwargs("en"), {
            'translations__language_code': 'en'
        })

    def test_get_index_queryset(self):
        idx_obj = QuestionIndex()
        # NOTE: This passes, but only serves to prove that the search_index
        # should be returning only objects that have the given translation!
        # TODO: Fix the above in search_indexes.py
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("en")],
            [self.question1.id, self.question2.id],
        )
        # This one is OK.
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("de")],
            [self.question1.id, self.question2.id],
        )

    def test_get_search_data(self):
        idx_obj = QuestionIndex()
        question1 = self.reload(self.question1, "en")
        search_data = idx_obj.get_search_data(question1, "en", None)
        self.assertEqual(search_data, "Test Question Test Answer")

        # This isn't working, but I think it proves that the search index
        # doesn't actually work translatedly.
        # TODO: Fix search_indexes.py
        # question1 = self.reload(self.question1, "de")
        # search_data = idx_obj.get_search_data(question1, "de", None)
        # self.assertEqual(search_data, "Testfrage Test Antwort")


class TestCategoryIndex(AldrynFaqTest):
    def test_get_title(self):
        # This search_index method always returns ""
        idx_obj = CategoryIndex()
        category1 = self.reload(self.category1, "en")
        self.assertEqual(idx_obj.get_title(category1), "")

    def test_get_index_kwargs(self):
        # This is a silly test, but is here for completeness.
        idx_obj = CategoryIndex()
        self.assertEqual(idx_obj.get_index_kwargs("en"), {
            'translations__language_code': 'en'
        })

    def test_get_index_queryset(self):
        idx_obj = CategoryIndex()
        # NOTE: This passes, but only serves to prove that the search_index
        # should be returning only objects that have the given translation!
        # TODO: Fix the above in search_indexes.py
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("en")],
            [self.category1.id, self.category2.id],
        )
        # This one is OK.
        self.assertEqualItems(
            [q.id for q in idx_obj.get_index_queryset("de")],
            [self.category1.id, self.category2.id],
        )

    def test_get_search_data(self):
        idx_obj = CategoryIndex()
        category1 = self.reload(self.category1, "en")
        search_data = idx_obj.get_search_data(category1, "en", None)
        self.assertEqual(search_data, "Example")

        # This isn't working, but I think it proves that the search index
        # doesn't actually work translatedly.
        # TODO: Fix search_indexes.py
        # category1 = self.reload(self.category1, "de")
        # search_data = idx_obj.get_search_data(category1, "de", None)
        # self.assertEqual(search_data, "Beispiel")



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

import six

from django.contrib.contenttypes.models import ContentType
from django.core.urlresolvers import reverse
from django.db import models
from django.utils.encoding import python_2_unicode_compatible
from django.utils.translation import override, ugettext_lazy as _
from cms.models.fields import PlaceholderField
from cms.models.pluginmodel import CMSPlugin
from cms.utils.i18n import get_current_language

from parler.models import TranslatableModel, TranslatedFields

from adminsortable.fields import SortableForeignKey
from adminsortable.models import Sortable

from djangocms_text_ckeditor.fields import HTMLField

from sortedm2m.fields import SortedManyToManyField

from .managers import CategoryManager, RelatedManager


def get_translation(obj, language_code):
    """This is an adapter from django-hvad.utils.get_translation(), a function
    to django-parler.models.get_translation() (a model instance method)."""
    if not obj or not hasattr(obj, "get_translation"):
        return None
    return obj.get_translation(language_code)


def get_slug_in_language(record, language):
    """This is an adapter from django-hvad's lazy_translation_getter."""
    if not record or not hasattr(record, "safe_translation_getter"):
        return None
    return record.safe_translation_getter(
        field="slug", language_code=language, default=None, )


@python_2_unicode_compatible
class Category(TranslatableModel):
    translations = TranslatedFields(
        name=models.CharField(max_length=255),
        slug=models.SlugField(verbose_name=_('Slug'), max_length=255),
    )

    objects = CategoryManager()

    class Meta:
        verbose_name = _('category')
        verbose_name_plural = _('categories')

    def __str__(self):
        if six.PY2:
            return self.safe_translation_getter('name', default=unicode(self.pk))
        else:
            return self.safe_translation_getter('name', default=str(self.pk))

    def model_type_id(self):
        return ContentType.objects.get_for_model(self.__class__).id

    def get_absolute_url(self, language=None):
        language = language or get_current_language()
        slug = get_slug_in_language(self, language)
        with override(language):
            if not slug:  # category not translated in given language
                return '/%s/' % language
            kwargs = {'category_slug': slug}
            return reverse('aldryn_faq:faq-category', kwargs=kwargs)


@python_2_unicode_compatible
class Question(TranslatableModel, Sortable):
    translations = TranslatedFields(
        title=models.CharField(_('Title'), max_length=255),
        answer_text=HTMLField(_('answer'))
    )
    category = SortableForeignKey(Category, related_name='questions')

    answer = PlaceholderField(
        'faq_question_answer', related_name='faq_questions')
    is_top = models.BooleanField(default=False)
    number_of_visits = models.PositiveIntegerField(default=0, editable=False)

    objects = RelatedManager()

    class Meta(Sortable.Meta):
        verbose_name = _('question')
        verbose_name_plural = _('questions')

    def __str__(self):
        return self.safe_translation_getter('title', default=str(self.pk))

    def model_type_id(self):
        return ContentType.objects.get_for_model(self.__class__).id

    def get_absolute_url(self, language=None):
        language = language or get_current_language()
        category = self.category
        try:
            translation = get_translation(self, language_code=language)
        except models.ObjectDoesNotExist:
            translation = None
        cat_slug = get_slug_in_language(category, language)
        if translation and cat_slug:
            with override(language):
                return reverse(
                    'aldryn_faq:faq-answer', args=(cat_slug, self.pk))
        else:
            return category.get_absolute_url(language)


class QuestionsPlugin(models.Model):
    questions = models.IntegerField(
        default=5,
        help_text=_('The number of questions to be displayed.')
    )

    def get_queryset(self):
        return Question.objects.filter_by_language(self.language)

    def get_questions(self):
        questions = self.get_queryset()
        return questions[:self.questions]

    class Meta:
        abstract = True


@python_2_unicode_compatible
class QuestionListPlugin(CMSPlugin):
    questions = SortedManyToManyField(Question)

    def __str__(self):
        return str(self.questions.count())

    def copy_relations(self, oldinstance):
        self.questions = oldinstance.questions.all()

    def get_questions(self):
        return self.questions.all()


class CategoryListPlugin(CMSPlugin):

    def copy_relations(self, oldinstance):
        for category in oldinstance.selected_categories.all():
            category.pk = None
            category.cms_plugin = self
            category.save()

    def get_categories(self):
        """
        By default, if no categories were chosen return all categories.
        Otherwise, return the chosen categories.
        """
        categories = Category.objects.get_categories(language=self.language)

        if self.selected_categories.exists():
            category_ids = self.selected_categories.values_list(
                'category__pk', flat=True)
            # categories is a list, and a sorted one so no need for another db
            # call.
            selected_categories = []
            for id in category_ids:
                category = next((x for x in categories if x.pk == id), None)
                if category:
                    selected_categories.append(category)
            return selected_categories
        return categories


class LatestQuestionsPlugin(CMSPlugin, QuestionsPlugin):

    def get_queryset(self):
        qs = super(LatestQuestionsPlugin, self).get_queryset()
        return qs.order_by('-id')


@python_2_unicode_compatible
class SelectedCategory(models.Model):
    category = models.ForeignKey(to=Category, verbose_name=_('category'))
    position = models.PositiveIntegerField(
        verbose_name=_('position'), blank=True, null=True)
    cms_plugin = models.ForeignKey(
        to=CategoryListPlugin, related_name='selected_categories')

    class Meta:
        ordering = ['position']
        verbose_name = _('Category')
        verbose_name_plural = _('Categories')

    def __str__(self):
        return self.category.name


class TopQuestionsPlugin(CMSPlugin, QuestionsPlugin):

    def get_queryset(self):
        qs = super(TopQuestionsPlugin, self).get_queryset()
        return qs.filter(is_top=True)


class MostReadQuestionsPlugin(CMSPlugin, QuestionsPlugin):

    def get_queryset(self):
        qs = super(MostReadQuestionsPlugin, self).get_queryset()
        return qs.order_by('-number_of_visits')



# -*- coding: utf-8 -*-

from __future__ import unicode_literals

from django.contrib.admin import TabularInline
from django.utils.translation import ugettext_lazy as _

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from cms.utils import get_language_from_request

from . import models
from .forms import QuestionListPluginForm


class FAQPlugin(CMSPluginBase):

    module = "FAQ"


class QuestionListPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/question_list.html'
    name = _('Question List')
    model = models.QuestionListPlugin
    form = QuestionListPluginForm

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class LatestQuestionsPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/latest_questions.html'
    name = _('Latest questions')
    model = models.LatestQuestionsPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class TopQuestionsPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/top_questions.html'
    name = _('Top questions')
    model = models.TopQuestionsPlugin

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class MostReadQuestionsPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/most_read_questions.html'
    name = _('Most read questions')
    model = models.MostReadQuestionsPlugin
    cache = False

    def render(self, context, instance, placeholder):
        context['instance'] = instance
        return context


class SelectedCategoryInline(TabularInline):
    model = models.SelectedCategory
    extra = 0

    def formfield_for_foreignkey(self, db_field, request=None, **kwargs):
        field = super(SelectedCategoryInline, self).formfield_for_foreignkey(
            db_field, request, **kwargs)

        if db_field.name == 'category':
            language = get_language_from_request(request)
            field.queryset = models.Category.objects.language(language)
        return field


class CategoryListPlugin(FAQPlugin):

    render_template = 'aldryn_faq/plugins/category_list.html'
    name = _('List of categories')
    model = models.CategoryListPlugin
    inlines = [SelectedCategoryInline]

    def render(self, context, instance, placeholder):
        context['categories'] = instance.get_categories()
        return context


plugin_pool.register_plugin(CategoryListPlugin)
plugin_pool.register_plugin(LatestQuestionsPlugin)
plugin_pool.register_plugin(QuestionListPlugin)
plugin_pool.register_plugin(TopQuestionsPlugin)
plugin_pool.register_plugin(MostReadQuestionsPlugin)



# -*- coding: utf-8 -*-
from setuptools import setup, find_packages
from aldryn_faq import __version__

REQUIREMENTS = [
    'django-hvad>=0.4.1',
    'django-admin-sortable',
    'djangocms-text-ckeditor',
    'django-sortedm2m',
    'unidecode',
]

CLASSIFIERS = [
    'Development Status :: 2 - Pre-Alpha',
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Programming Language :: Python',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
]

setup(
    name='aldryn-faq',
    version=__version__,
    description='FAQ addon for django CMS',
    author='Divio AG',
    author_email='info@divio.ch',
    url='https://github.com/aldryn/aldryn-faq',
    packages=find_packages(),
    license='LICENSE.txt',
    platforms=['OS Independent'],
    install_requires=REQUIREMENTS,
    classifiers=CLASSIFIERS,
    include_package_data=True,
    zip_safe=False
)
